'use strict';

// use Polyfill for util.promisify in node versions < v8
var promisify = require('util.promisify');

var fs = require('fs')
var path = require('path')

var fsExists = promisify(fs.exists)
var fsWriteFile = promisify(fs.writeFile)

class DevServeUpdateWebpackPlugin {
  constructor(options) {
    const userOptions = options || {}

    const defaultOptions = {
      // 开发环境排除热更新生成的 .hot.update.js 文件
      exceptHotUpdateJs: true
    }

    this.options = Object.assign(defaultOptions, userOptions)

    this.varsion = DevServeUpdateWebpackPlugin.version
  }

  // 开发环境 更新生成的文件
  devUpdate(info) {
    var srcs = info.srcs
    var output = this.options.output
    var appName = this.options.appName

    if (!output) {
      console.warn(`\nError：Option of 'output' is required.`)
      return
    }

    if (!appName) {
      console.warn(`\nError：Option of 'appName' is required.`)
      return
    }

    fsExists(output)
      .then(function (exists){
        var superServeConfig = {}

        if (exists) {
          try {
            superServeConfig = require(output);
          }
          catch (err) {
            console.error(err)
            return
          }
        }

        return Object.assign(superServeConfig, {
          [appName]: {
            src: srcs,
            timestamp: Date.now()
          }
        })
      })
      .then(function (config){
        return fsWriteFile(output, JSON.stringify(config, null, 2))
      })
      .then(function () {
        let consoleStr = `\nSUCCESS: Uptade super serve config success.\n`

        for (var index = 0; index < srcs.length; index++) {
          consoleStr += `- ${srcs[index]}\n`
        }

        console.log(consoleStr);
      })
      .catch(function (err) {
        throw err;
      })
  }

  apply(compiler) {
    var self = this
    var devHost = "http://localhost"

    if (compiler.options.mode === 'development'){
      compiler.hooks.afterEmit.tap(
        'DevServeUpdateWebpackPlugin',

        function (compilation) {

          console.log(compilation.options.output.publicPath)

          var assets = compilation.assets;
          var publicPath = compilation.options.output.publicPath || '/';
          var devServerPort = self.options.devServerPort

          if (!devServerPort) {
            console.warn(`Error：Option of 'devServerPort' is required`)
            return
          }

          var js = Object.keys(assets)
            // todo -> 优化正则表达式
            .filter(name => name.indexOf('.hot-update') < 0 && /.js$/.test(name))
            .map(name => path.join(`${devHost}:${devServerPort}`, publicPath, name ));

          self.devUpdate({ srcs: js });
        }
      )
    }
  }
}

/* 
  version
*/
DevServeUpdateWebpackPlugin.version = 1

module.exports = DevServeUpdateWebpackPlugin
